#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"

#include "zmbuf.h"


#ifndef arraylen
#define arraylen(a)	(sizeof (a) / sizeof (a)[0])
#endif


typedef struct zmbuf *Zoned__Message;


static enum ZonedMessage_PR zmbuf_typestr(const char *type) {
	if (strEQ(type, "helo"))
		return ZonedMessage_PR_helo;
	if (strEQ(type, "ping"))
		return ZonedMessage_PR_ping;
	if (strEQ(type, "data"))
		return ZonedMessage_PR_data;
	if (strEQ(type, "edit"))
		return ZonedMessage_PR_edit;

	return 0;
} /* zmbuf_typestr() */


static const char *zmbuf_strtype(enum ZonedMessage_PR type) {
	switch (type) {
	case ZonedMessage_PR_helo:
		return "helo";
	case ZonedMessage_PR_ping:
		return "ping";
	case ZonedMessage_PR_data:
		return "data";
	case ZonedMessage_PR_edit:
		return "edit";
	default:
		return 0;
	}
} /* zmbuf_strtype() */


static enum ZonedEditMode zmbuf_editmodestr(const char *mode) {
	if (strEQ(mode, "set"))
		return ZonedEditMode_editModeSet;
	if (strEQ(mode, "add"))
		return ZonedEditMode_editModeAdd;
	if (strEQ(mode, "del"))
		return ZonedEditMode_editModeDel;

	return -1;
} /* zmbuf_editmodestr() */


static const char *zmbuf_streditmode(enum ZonedEditMode mode) {
	switch (mode) {
	case ZonedEditMode_editModeSet:
		return "set";
	case ZonedEditMode_editModeAdd:
		return "add";
	case ZonedEditMode_editModeDel:
		return "del";
	default:
		return 0;
	}
} /* zmbuf_streditmode() */


static struct zmbuf *zmbuf__new_helo(int nopts, SV *opts[]) {
	const char *host	= 0;
	int port		= -1;
	struct zmbuf *mb;
	int i;

	for (i = 0; i < nopts - 1; i += 2) {
		char *optnam	= SvPV_nolen(opts[i]);

		if (strEQ(optnam, "host")) {
			host	= SvPV_nolen(opts[i + 1]);
		} else if (strEQ(optnam, "port")) {
			port	= SvIV(opts[i + 1]);

			if (port < 0 || port > 65535)
				croak("invalid port: %d", port);
		}
	}

	if (host == 0)
		croak("missing host");
	else if (port == -1)
		croak("missing port");

	if (0 == (mb = zmbuf_helo(host, port)))
		croak("unable to allocate message: %s", strerror(errno));

	return mb;
} /* zmbuf__new_helo() */


static struct zmbuf *zmbuf__new_ping(int nopts, SV *opts[]) {
	time_t now	= -1;
	struct zmbuf *mb;
	int i;

	for (i = 0; i < nopts - 1; i += 2) {
		char *optnam	= SvPV_nolen(opts[i]);

		if (strEQ(optnam, "now"))
			now	= SvIV(opts[i + 1]);
	}

	if (now == -1)
		croak("missing `now' timestamp");

	if (0 == (mb = zmbuf_ping(now)))
		croak("unable to allocate message: %s", strerror(errno));

	return mb;
} /* zmbuf__new_ping() */


static struct zmbuf *zmbuf__new_data(int nopts, SV *opts[]) {
	croak("Zoned::Message::Data type not supported");
} /* zmbuf__new_data() */


static struct zmbuf *zmbuf__new_edit(int nopts, SV *opts[]) {
	const char *zone	= 0;
	enum ZonedEditMode mode	= ZonedEditMode_editModeSet;
	struct zmbuf *mb;
	int i;

	for (i = 0; i < nopts - 1; i += 2) {
		char *optnam	= SvPV_nolen(opts[i]);

		if (strEQ(optnam, "zone")) {
			zone	= SvPV_nolen(opts[i + 1]);
		} else if (strEQ(optnam, "mode")) {
			if (-1 == (mode = zmbuf_editmodestr(SvPV_nolen(opts[i + 1]))))
				croak("invalid edit mode: %s", SvPV_nolen(opts[i + 1]));
		}
	}

	if (zone == 0)
		croak("missing zone");

	if (0 == (mb = zmbuf_edit(zone, mode)))
		croak("unable to allocate message: %s", strerror(errno));

	return mb;
} /* zmbuf__new_edit() */


static void zmbuf__add_any(struct zmbuf *mb, int argc, SV *args[], const char *host, int ttl) {
	const char *rtype	= 0;
	const char *rdata	= 0;
	int i;

	for (i = 0; i < argc - 1; i += 2) {
		char *optnam	= SvPV_nolen(args[i]);

		if (strEQ(optnam, "rtype"))
			rtype	= SvPV_nolen(args[i + 1]);
		if (strEQ(optnam, "rdata"))
			rdata	= SvPV_nolen(args[i + 1]);
	}

	if (rtype == 0)
		croak("missing rtype");
	if (rdata == 0)
		croak("missing rdata");

	if (0 != zmbuf_edit_any(mb, host, ttl, rtype, rdata))
		croak("unable to allocate object: %s", strerror(errno));

	return /* void */;
} /* zmbuf__add_any() */


static void zmbuf__add_a(struct zmbuf *mb, int argc, SV *args[], const char *host, int ttl) {
	const char *address	= 0;
	int i;

	for (i = 0; i < argc - 1; i += 2) {
		char *optnam	= SvPV_nolen(args[i]);

		if (strEQ(optnam, "address"))
			address	= SvPV_nolen(args[i + 1]);
	}

	if (address == 0)
		croak("missing address");

	if (0 != zmbuf_edit_a(mb, host, ttl, address))
		croak("unable to allocate object: %s", strerror(errno));

	return /* void */;
} /* zmbuf__add_a() */


static void zmbuf__add_srv(struct zmbuf *mb, int argc, SV *args[], const char *host, int ttl) {
	const char *target	= 0;
	int priority		= -1;
	int weight		= -1;
	int port		= -1;
	int i;

	for (i = 0; i < argc - 1; i += 2) {
		char *optnam	= SvPV_nolen(args[i]);

		if (strEQ(optnam, "target"))
			target		= SvPV_nolen(args[i + 1]);
		else if (strEQ(optnam, "priority"))
			priority	= SvIV(args[i + 1]);
		else if (strEQ(optnam, "weight"))
			weight		= SvIV(args[i + 1]);
		else if (strEQ(optnam, "port"))
			port		= SvIV(args[i + 1]);
	}

	if (target == 0)
		croak("missing target");
	else if (priority < 0)
		croak("missing or invalid priority: %d", priority);
	else if (weight < 0)
		croak("missing or invalid weight: %d", weight);
	else if (port < 0 || port > 65535)
		croak("missing or invalid port: %d", port);

	if (0 != zmbuf_edit_srv(mb, host, ttl, priority, weight, port, target))
		croak("unable to allocate object: %s", strerror(errno));

	return /* void */;
} /* zmbuf__add_srv() */



MODULE = Zoned			PACKAGE = Zoned::Message

PROTOTYPES: ENABLE


Zoned::Message
new(class, ...)
	char *class
ALIAS:
	Zoned::Message::Helo::new	= ZonedMessage_PR_helo
	Zoned::Message::Ping::new	= ZonedMessage_PR_ping
	Zoned::Message::Data::new	= ZonedMessage_PR_data
	Zoned::Message::Edit::new	= ZonedMessage_PR_edit
PREINIT:
	enum ZonedMessage_PR type;
	static typeof(&zmbuf__new_helo) constr[]	= {
		[ZonedMessage_PR_helo]	= &zmbuf__new_helo,
		[ZonedMessage_PR_ping]	= &zmbuf__new_ping,
		[ZonedMessage_PR_data]	= &zmbuf__new_data,
		[ZonedMessage_PR_edit]	= &zmbuf__new_edit,
	};
	int i, optind;
	SV *opts[32];
CODE:
	if (ix == 0) {
		if (items < 2 || !SvPOK(ST(1)))
			croak("must specify message type");

		type	= zmbuf_typestr(SvPV_nolen(ST(1)));
		optind	= 2;
	} else {
		type	= ix;
		optind	= 1;
	}

	for (i = 0; i < arraylen(opts) - 1 && optind < items - 1; i += 2, optind +=2) {
		opts[i + 0]	= ST(optind + 0);
		opts[i + 1]	= ST(optind + 1);
	}

	if (type >= 0 && type < arraylen(constr) && 0 != constr[type]) {
		RETVAL	= constr[type](i, opts);
	} else
		croak("invalid message type: %d", type);
OUTPUT:
	RETVAL


Zoned::Message
load(class, ...)
	char *class
ALIAS:
	decode		= 1
PREINIT:
	const char *encoding	= "der";
	SV *src;
	STRLEN len;
	void *p;
CODE:
	if (items > 2) {
		encoding	= SvPV_nolen(ST(1));
		src		= ST(2);
	} else
		src		= ST(1);

	if (0 == (RETVAL = zmbuf_alloc()))
		croak("unable to allocate object: %s", strerror(errno));

	p	= SvPVbyte(src, len);

	if (strEQ(encoding, "ber") || strEQ(encoding, "der")) {
		(void)zmsg_decode_ber(&RETVAL->msg, p, len);
	} else if (strEQ(encoding, "xer")) {
		(void)zmsg_decode_xer(&RETVAL->msg, p, len);
	}

	if (0 == RETVAL->msg) {
		zmbuf_free(RETVAL), RETVAL = 0;

		croak("unable to allocate object: %s", strerror(errno));
	}
OUTPUT:
	RETVAL


const char *
type(mb)
	Zoned::Message mb
CODE:
	if (0 == (RETVAL = zmbuf_strtype(mb->msg->present)))
		croak("unknown type: %d", mb->msg->present);
OUTPUT:
	RETVAL


const char *
mode(mb)
	Zoned::Message mb
PREINIT:
	long mode;
CODE:
	if (mb->msg->present != ZonedMessage_PR_edit)
		croak("expected ZonedEdit message");
	if (0 != asn_INTEGER2long(&mb->msg->edit.mode, &mode))
		croak("unable to determine mode: %s", strerror(errno));
	if (0 == (RETVAL = zmbuf_streditmode(mode)))
		croak("unknown mode: %d", mode);
OUTPUT:
	RETVAL


SV *
encode(mb, encoding = "der")
	Zoned::Message mb
	const char *encoding
PREINIT:
	void *p		= 0;
	size_t len	= 0;
	asn_enc_rval_t rc;
CODE:
	if (strEQ(encoding, "ber") || strEQ(encoding, "der")) {
		rc	= zmsg_encode_der(&p, &len, mb->msg);
	} else if (strEQ(encoding, "xer")) {
		rc	= zmsg_encode_xer(&p, &len, mb->msg);
	} else
		croak("invalid encoding: %s", encoding);

	if (p == 0)
		croak("unable to encode message: %s", (rc.failed_type != 0 && rc.failed_type->name != 0)? rc.failed_type->name : "?");

	RETVAL	= newSVpvn(p, len);

	free(p), p = 0;
OUTPUT:
	RETVAL


void
addAny(mb, ...)
	Zoned::Message mb
ALIAS:
	addA	= 1
	addSRV	= 2
PREINIT:
	static typeof(&zmbuf__add_any) constr[]	= {
		[0]	= &zmbuf__add_any,
		[1]	= &zmbuf__add_a,
		[2]	= &zmbuf__add_srv,
	};
	int i, j;
	SV *args[32];
	const char *host	= 0;
	int ttl			= -1;
CODE:
	for (i = 0, j = 1; i < arraylen(args) - 1 && j < items - 1; i += 2, j +=2) {
		SV *optnam	= ST(j + 0);
		SV *optval	= ST(j + 1);

		args[i + 0]	= optnam;
		args[i + 1]	= optval;

		if (strEQ(SvPV_nolen(optnam), "host"))
			host	= SvPV_nolen(optval);
		else if (strEQ(SvPV_nolen(optnam), "ttl"))
			ttl	= SvIV(optval);
	}

	if (host == 0)
		croak("missing host");

	if (ttl < 0)
		croak("invalid TTL: %d", ttl);

	if (ix < 0 || ix >= arraylen(constr) || 0 == constr[ix])
		croak("unknown RR type constructor: %d", ix);

	constr[ix](mb, i, args, host, ttl);
	

void
DESTROY(mb)
	Zoned::Message mb
CODE:
	zmbuf_free(mb), mb = 0;

