/* ==========================================================================
 * Zoned - BIND DLZ Replicator
 * --------------------------------------------------------------------------
 * Copyright (C) 2007  William Ahern
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * ==========================================================================
 */
#include <sys/types.h>		/* socklen_t */
#include <sys/socket.h>		/* struct sockaddr */

#include <evnet/socket.h>	/* struct socket_name socket_name_init_addr */

#include "log.h"
#include "options.h"
#include "znode.h"
#include "listen.h"


static void listen_accept(struct socket *s0, struct socket *s, enum socket_errno s_errno, void *arg) {
	struct znode *z;

	if (s == 0)
		panic("socket_accept: %s", (s_errno == SOCKET_ESYSTEM)? strerror(errno) : socket_strerror(s_errno));

	if (0 == (z = znode_open(s)))
		panic("unable to open znode");

	znode_start(z);

	return /* void */;
} /* listen_accept() */


void listen_init(void) {
	struct sockaddr *sa;
	struct socket_name s_name;
	struct socket_options s_opts;
	enum socket_errno s_errno;
	struct socket *s;

	luaZ_getglobal(opts.L, "proc", "listen");

	lua_pushnil(opts.L);

	while (0 != lua_next(opts.L, -2) && 0 != (sa = luaL_checkudata(opts.L, -1, "sockaddr"))) {
		s_opts			= socket_defaults;
		s_opts.sa_reuseaddr	= 1;
		s_opts.so_nonblock	= 1;
		s_opts.sin_resolv	= 0;

		socket_name_init_addr(&s_name, sa, SOCKET_SA_LENOF(sa));

		if (0 == (s = socket_open(&s_name, &s_opts, 0, 0, &s_errno)))
			panic("socket_open: %s", socket_strerror(s_errno));

		socket_accept(s, &listen_accept, 0, 0);

		lua_pop(opts.L, 1);
	}

	lua_pop(opts.L, 1);

	return /* void */;
} /* listen_init() */
