/* ==========================================================================
 * pidfile.h - 'nix "pid file" implementation
 * --------------------------------------------------------------------------
 * Copyright (c) 2007  William Ahern
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ==========================================================================
 */
#ifndef PIDFILE_H
#define PIDFILE_H

#include <sys/types.h>		/* mode_t pid_t */

#define PIDFILE_VERSION		2007041601UL

#define PIDFILE_VOID_PID	0


struct pidfile;


extern const struct pidfile_options {
	mode_t mask;

	struct {
		void *arg;

		int (*nolock)(struct pidfile *, void *);
		int (*onlock)(struct pidfile *, void *);
		int (*unlock)(struct pidfile *, void *);
	} cb;
} pidfile_defaults;


struct pidfile *pidfile_open(const char *, const struct pidfile_options *);

void pidfile_close(struct pidfile *, int);

int pidfile_lock(struct pidfile *);

int pidfile_trylock(struct pidfile *);

pid_t pidfile_getpid(struct pidfile *);

int pidfile_setpid(struct pidfile *, pid_t);


#endif /* PIDFILE_H */
