/* ==========================================================================
 * zmbuf.h - Zoned - BIND DLZ Replicator
 * --------------------------------------------------------------------------
 * Copyright (C) 2007  William Ahern
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * ==========================================================================
 */
#ifndef ZMBUF_H
#define ZMBUF_H

#include <stddef.h>		/* size_t */

#include <time.h>		/* time_t */

#include <sys/queue.h>		/* LIST SIMPLEQ */

#include <db.h>			/* DBT DB_LSN */

#include "ZonedMessage.h"	/* asn_dec_rval_t asn_enc_rval_t struct ZonedMessage */


struct zmbuf {
	struct ZonedMessage *msg;

	struct {
		LIST_HEAD(, zmbuf) *refs, lh;
		LIST_ENTRY(zmbuf) le;
	} ref;

	SIMPLEQ_ENTRY(zmbuf) sqe;
}; /* zmbuf  */


#define zmsg_decode(...)	zmsg_decode_ber(__VA_ARGS__)

#define zmsg_encode(...)	zmsg_encode_der(__VA_ARGS__)

asn_dec_rval_t zmsg_decode_xer(struct ZonedMessage **, const void *, size_t);

asn_enc_rval_t zmsg_encode_xer(void **, size_t *, struct ZonedMessage *);

asn_dec_rval_t zmsg_decode_ber(struct ZonedMessage **, const void *, size_t);

asn_enc_rval_t zmsg_encode_der(void **, size_t *, struct ZonedMessage *);

void zmsg_free(struct ZonedMessage *);

struct ZonedMessage *zmsg_alloc(void);

struct ZonedMessage *zmsg_new(enum ZonedMessage_PR);

void zmbuf_free(struct zmbuf *);

struct zmbuf *zmbuf_alloc(void);

struct zmbuf *zmbuf_new(enum ZonedMessage_PR);

struct zmbuf *zmbuf_dup(struct zmbuf *);

struct zmbuf *zmbuf_helo(const char *, unsigned short);

struct zmbuf *zmbuf_ping(time_t);

struct zmbuf *zmbuf_data(const DBT *, const DBT *, const DB_LSN *);

struct zmbuf *zmbuf_edit(const char *, enum ZonedEditMode);

int zmbuf_edit_any(struct zmbuf *, const char *, unsigned, const char *, const char *);

int zmbuf_edit_a(struct zmbuf *, const char *, unsigned, const char *);

int zmbuf_edit_srv(struct zmbuf *, const char *, unsigned, unsigned, unsigned, unsigned short, const char *);


#endif /* ZMBUF_H */

